using Microsoft.AspNetCore.Mvc;
using InternshipSystem.Core;
using System.Threading.Tasks;
using System.Collections.Generic;
using InternshipSystem.Repository;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{

    [ApiController]
    [Route("Internship")]
    public class InternshipController : ControllerBase {
        private readonly InternshipDbContext _context;

        public InternshipController(InternshipDbContext context)
        {
            this._context = context;
        }

        [HttpPost]
        public async Task<ActionResult<Internship>> CreateInternship(Internship internship) 
        {
            this._context.Internships.Add(internship);
            await this._context.SaveChangesAsync();

            return CreatedAtAction(nameof(GetInternship), new { id = internship.Id }, internship);
        }

        [HttpGet]
        public async Task<ActionResult<IEnumerable<Internship>>> GetAllInternships() => 
            await this._context.Internships.ToListAsync();

        [HttpGet("{id}")]
        public async Task<ActionResult<Internship>> GetInternship(int id) 
        {
            var internship = await this._context.Internships.FindAsync(id);

            if(internship == null) {
                return NotFound();
            }

            return internship;
        }

        //TODO: Refactor
        [HttpPut("{id}")]
        public async Task<IActionResult> UpdateInternship(int id, Internship internship) 
        {
            if (id != internship.Id)
            {
                return BadRequest();
            }

            var internshipItem = await this._context.Internships.FindAsync(id);
            if (internshipItem == null)
            {
                return NotFound();
            }

            try
            {
                await this._context.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException) when (!InternshipsExists(id))
            {
                return NotFound();
            }

            return NoContent();
        }

        private bool InternshipsExists(int id) =>
            this._context.Internships.Find(id) != null;

        [HttpDelete("{id}")]
        public async Task<IActionResult> DeleteInternship(int id) {
            var internshipItem = await this._context.Internships.FindAsync(id);

            if (internshipItem == null)
            {
                return NotFound();
            }

            this._context.Internships.Remove(internshipItem);
            await _context.SaveChangesAsync();

            return NoContent();
        }
    }
}